@@ -149,7 +149,7 @@ class ParallelGenerator:
         self.model = model
         self.config = config
         self.use_kv_cache = use_kv_cache
-        self.use_cuda_graphs = use_cuda_graphs
+        self.cuda_graphs_initialized = False  # Track if CUDA graphs are actually set up
 
         # Setup KV cache if enabled
         if use_kv_cache:
@@ -162,10 +162,27 @@ class ParallelGenerator:
 
         # Compile model for faster inference
         if config.use_torch_compile and hasattr(torch, "compile"):
-            self.model = torch.compile(self.model)
+            # Check for GPU capability if using CUDA
+            can_compile = True
+            if torch.cuda.is_available():
+                device = next(model.parameters()).device
+                if device.type == "cuda":
+                    # Get device capability
+                    capability = torch.cuda.get_device_capability(device)
+                    if capability[0] < 7:
+                        print(f"Warning: GPU capability {capability[0]}.{capability[1]} is too old for torch.compile (requires >= 7.0). Disabling compilation.")
+                        can_compile = False
+            
+            if can_compile:
+                self.model = torch.compile(self.model)
+                # torch.compile and CUDA graphs are incompatible
+                if use_cuda_graphs:
+                    print("Warning: CUDA graphs disabled when torch.compile is enabled (incompatible)")
+                    use_cuda_graphs = False
 
         if use_cuda_graphs:
             self._setup_cuda_graphs()
+            self.cuda_graphs_initialized = True
 
     def _setup_cuda_graphs(self):
         """Setup CUDA graphs for maximum throughput"""
@@ -214,12 +231,20 @@ class ParallelGenerator:
         max_new_tokens = max_new_tokens or self.config.max_new_tokens
         device = prompt_tokens.device
 
-        # Validate prompt tokens against vocab size
-        if prompt_tokens.max() >= self.model.config.vocab_size:
-            raise ValueError(
-                f"Prompt contains token IDs greater than model vocabulary size "
-                f"({self.model.config.vocab_size}). This will cause CUDA errors."
+        # Validate prompt tokens against vocab size and resize if necessary
+        max_token_id = prompt_tokens.max().item()
+        if max_token_id >= self.model.config.vocab_size:
+            print(
+                f"Warning: Prompt contains token IDs ({max_token_id}) greater than model vocabulary size "
+                f"({self.model.config.vocab_size}). Resizing model embeddings..."
             )
+            # Resize embeddings to the next multiple of 64 for performance
+            new_vocab_size = ((max_token_id + 1 + 63) // 64) * 64
+            self.model.resize_token_embeddings(new_vocab_size)
+            print(f"Model embeddings resized to {new_vocab_size}")
+            
+            # Update config
+            self.model.config.vocab_size = new_vocab_size
 
         # Initialize full sequence with MASK tokens (using 0 as mask token)
         total_len = prompt_len + max_new_tokens
@@ -251,7 +276,7 @@ class ParallelGenerator:
             timestep = torch.full((batch_size,), current_step, device=device, dtype=torch.long)
 
             # Forward pass - predict all positions simultaneously
-            if self.use_cuda_graphs and generated.shape[1] <= self.static_input.shape[1]:
+            if self.cuda_graphs_initialized and generated.shape[1] <= self.static_input.shape[1]:
                 # Use CUDA graph for fixed-size inputs
                 self.static_input[:batch_size, :total_len].copy_(generated)
                 self.static_timestep[:batch_size].copy_(timestep)
