@@ -20,6 +20,7 @@ import functools
 from typing import Optional, Dict, Any
 from pathlib import Path
 import logging
+import datetime
 from tqdm import tqdm
 import wandb
 
@@ -72,23 +73,66 @@ class DistributedTrainer:
 
     def _init_distributed(self):
         """Initialize distributed training or single-device training"""
-        # Check if we're in a distributed environment
-        self.is_distributed = (
-            "LOCAL_RANK" in os.environ or
-            "RANK" in os.environ or
-            "WORLD_SIZE" in os.environ or
-            torch.cuda.device_count() > 1
-        )
+        # Configure logging if not already configured
+        if not logging.getLogger().handlers:
+            logging.basicConfig(
+                format='%(asctime)s - %(levelname)s - %(name)s - %(message)s',
+                datefmt='%m/%d/%Y %H:%M:%S',
+                level=logging.INFO
+            )
 
-        if self.is_distributed:
-            # Initialize distributed training
+        # Check for explicit distributed environment (e.g., torchrun)
+        has_explicit_env = (
+            "RANK" in os.environ and
+            "WORLD_SIZE" in os.environ
+        )
+        
+        # Check for multi-GPU but no distributed env (Kaggle/Colab scenario)
+        num_gpus = torch.cuda.device_count() if torch.cuda.is_available() else 0
+        has_multi_gpu = num_gpus > 1 and not has_explicit_env
+        
+        if has_explicit_env:
+            # Explicit distributed environment (torchrun, etc.)
+            self.is_distributed = True
             if not dist.is_initialized():
                 try:
-                    dist.init_process_group(backend=self.train_config.distributed_backend)
+                    # Add timeout to fail fast if something is wrong
+                    dist.init_process_group(
+                        backend=self.train_config.distributed_backend,
+                        timeout=datetime.timedelta(seconds=30)
+                    )
                 except Exception as e:
                     logger.warning(f"Failed to initialize distributed training: {e}")
                     logger.info("Falling back to single-device training")
                     self.is_distributed = False
+                    
+        elif has_multi_gpu:
+            # Auto-initialize for Kaggle/Colab with multiple GPUs
+            logger.info(f"🚀 Detected {num_gpus} GPUs. Auto-initializing distributed training...")
+            logger.info("💡 Tip: For more control, use: torchrun --nproc_per_node={num_gpus} script.py")
+            
+            # Set up environment variables for torch.distributed
+            os.environ.setdefault("MASTER_ADDR", "localhost")
+            os.environ.setdefault("MASTER_PORT", "12355")
+            os.environ.setdefault("RANK", "0")
+            os.environ.setdefault("WORLD_SIZE", str(num_gpus))
+            os.environ.setdefault("LOCAL_RANK", "0")
+            
+            self.is_distributed = True
+            try:
+                # Add timeout to fail fast
+                dist.init_process_group(
+                    backend="nccl", 
+                    init_method="env://",
+                    timeout=datetime.timedelta(seconds=30)
+                )
+            except Exception as e:
+                logger.warning(f"Failed to auto-initialize distributed training: {e}")
+                logger.info("Falling back to single-device training")
+                self.is_distributed = False
+        else:
+            # Single device
+            self.is_distributed = False
 
         if self.is_distributed:
             self.rank = dist.get_rank()
@@ -97,7 +141,7 @@ class DistributedTrainer:
             torch.cuda.set_device(self.local_rank)
             self.device = torch.device(f"cuda:{self.local_rank}")
             if self.rank == 0:
-                logger.info(f"Initialized distributed training with {self.world_size} GPUs")
+                logger.info(f"✅ Initialized distributed training with {self.world_size} GPUs")
         else:
             # Single device training
             self.rank = 0
@@ -108,28 +152,35 @@ class DistributedTrainer:
                 torch.cuda.set_device(0)
             else:
                 self.device = torch.device("cpu")
-            logger.info(f"Initialized single-device training on {self.device}")
+            logger.info(f"✅ Initialized single-device training on {self.device}")
 
     def _setup_distributed_model(self):
         """Setup model with FSDP or DeepSpeed"""
+        logger.info("Setting up model on devices...")
         self.model = self.model.to(self.device)
 
         if not self.is_distributed:
             # Single device - no distributed wrapper needed
-            logger.info("Using single-device model (no distributed wrapper)")
+            logger.info("✓ Model ready on single device")
             return
 
         if self.train_config.use_deepspeed:
+            logger.info("Initializing DeepSpeed ZeRO...")
             self._setup_deepspeed()
+            logger.info("✓ DeepSpeed initialized")
         elif self.train_config.use_fsdp:
+            logger.info("Initializing FSDP...")
             self._setup_fsdp()
+            logger.info("✓ FSDP initialized")
         else:
             # Standard DDP
+            logger.info("Initializing DDP...")
             self.model = torch.nn.parallel.DistributedDataParallel(
                 self.model,
                 device_ids=[self.local_rank],
                 output_device=self.local_rank,
             )
+            logger.info("✓ DDP initialized")
 
     def _setup_fsdp(self):
         """Setup Fully Sharded Data Parallel (FSDP2)"""
